@@ -388,7 +388,8 @@ static int __file_invalidate_cache(struct thread_data *td, struct fio_file *f,
 	if (f->mmap_ptr) {
 		ret = posix_madvise(f->mmap_ptr, f->mmap_sz, POSIX_MADV_DONTNEED);
 #ifdef FIO_MADV_FREE
-		(void) posix_madvise(f->mmap_ptr, f->mmap_sz, FIO_MADV_FREE);
+		if (f->filetype == FIO_TYPE_BD)
+			(void) posix_madvise(f->mmap_ptr, f->mmap_sz, FIO_MADV_FREE);
 #endif
 	} else if (f->filetype == FIO_TYPE_FILE) {
 		ret = posix_fadvise(f->fd, off, len, POSIX_FADV_DONTNEED);
@@ -443,6 +444,7 @@ int generic_close_file(struct thread_data fio_unused *td, struct fio_file *f)
 		f->shadow_fd = -1;
 	}
 
+	f->engine_data = 0;
 	return ret;
 }
 
@@ -517,6 +519,13 @@ int generic_open_file(struct thread_data *td, struct fio_file *f)
 		goto skip_flags;
 	if (td->o.odirect)
 		flags |= OS_O_DIRECT;
+	if (td->o.oatomic) {
+		if (!FIO_O_ATOMIC) {
+			td_verror(td, EINVAL, "OS does not support atomic IO");
+			return 1;
+		}
+		flags |= OS_O_DIRECT | FIO_O_ATOMIC;
+	}
 	if (td->o.sync_io)
 		flags |= O_SYNC;
 	if (td->o.create_on_open)
@@ -582,7 +591,7 @@ open_again:
 			 * work-around a "feature" on Linux, where a close of
 			 * an fd that has been opened for write will trigger
 			 * udev to call blkid to check partitions, fs id, etc.
-			 * That polutes the device cache, which can slow down
+			 * That pollutes the device cache, which can slow down
 			 * unbuffered accesses.
 			 */
 			if (f->shadow_fd == -1)
