@@ -49,6 +49,11 @@ struct io_log {
 
 	unsigned int log_type;
 
+	/*
+	 * If we fail extending the log, stop collecting more entries.
+	 */
+	unsigned int disabled;
+
 	/*
 	 * Windowed average, for logging single entries average over some
 	 * period of time.
@@ -118,7 +123,8 @@ extern void add_slat_sample(struct thread_data *, enum fio_ddir, unsigned long,
 				unsigned int);
 extern void add_bw_sample(struct thread_data *, enum fio_ddir, unsigned int,
 				struct timeval *);
-extern void add_iops_sample(struct thread_data *, enum fio_ddir, struct timeval *);
+extern void add_iops_sample(struct thread_data *, enum fio_ddir, unsigned int,
+				struct timeval *);
 extern void init_disk_util(struct thread_data *);
 extern void update_rusage_stat(struct thread_data *);
 extern void setup_log(struct io_log **, unsigned long, int);
