@@ -7,6 +7,7 @@
 #include <sys/shm.h>
 #include <sys/stat.h>
 #include <errno.h>
+#include <winsock2.h>
 #include <windows.h>
 #include <psapi.h>
 #include <stdlib.h>
@@ -17,6 +18,10 @@
 
 #include "windows/posix.h"
 
+#ifndef PTHREAD_STACK_MIN
+#define PTHREAD_STACK_MIN 65535
+#endif
+
 #define FIO_HAVE_ODIRECT
 #define FIO_HAVE_CPU_AFFINITY
 #define FIO_HAVE_CHARDEV_SIZE
@@ -37,9 +42,6 @@
 
 typedef DWORD_PTR os_cpu_mask_t;
 
-#define CLOCK_REALTIME	1
-#define CLOCK_MONOTONIC	2
-
 #define _SC_PAGESIZE			0x1
 #define _SC_NPROCESSORS_ONLN	0x2
 #define _SC_PHYS_PAGES			0x4
@@ -112,6 +114,8 @@ static inline int blockdev_size(struct fio_file *f, unsigned long long *bytes)
 {
 	int rc = 0;
 	HANDLE hFile;
+	GET_LENGTH_INFORMATION info;
+	DWORD outBytes;
 
 	if (f->hFile == NULL) {
 		hFile = CreateFile(f->file_name, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE,
@@ -120,10 +124,6 @@ static inline int blockdev_size(struct fio_file *f, unsigned long long *bytes)
 		hFile = f->hFile;
 	}
 
-	GET_LENGTH_INFORMATION info;
-	DWORD outBytes;
-	LARGE_INTEGER size;
-	size.QuadPart = 0;
 	if (DeviceIoControl(hFile, IOCTL_DISK_GET_LENGTH_INFO, NULL, 0, &info, sizeof(info), &outBytes, NULL))
 		*bytes = info.Length.QuadPart;
 	else
