@@ -8,6 +8,8 @@
 #include <unistd.h>
 #include <stdlib.h>
 
+#include "../arch/arch.h"
+
 enum {
 	os_linux = 1,
 	os_aix,
@@ -79,7 +81,7 @@ typedef unsigned long os_cpu_mask_t;
 #endif
 
 #ifndef FIO_HAVE_IOPRIO
-#define ioprio_set(which, who, prio)	(0)
+#define ioprio_set(which, who, prioclass, prio)	(0)
 #endif
 
 #ifndef FIO_HAVE_ODIRECT
@@ -88,6 +90,12 @@ typedef unsigned long os_cpu_mask_t;
 #define OS_O_DIRECT			O_DIRECT
 #endif
 
+#ifdef OS_O_ATOMIC
+#define FIO_O_ATOMIC			OS_O_ATOMIC
+#else
+#define FIO_O_ATOMIC			0
+#endif
+
 #ifndef FIO_HAVE_HUGETLB
 #define SHM_HUGETLB			0
 #define MAP_HUGETLB			0
