@@ -243,15 +243,13 @@ out:
 int calc_lat(struct io_stat *is, unsigned long *min, unsigned long *max,
 	     double *mean, double *dev)
 {
-	double n = is->samples;
+	double n = (double) is->samples;
 
-	if (is->samples == 0)
+	if (n == 0)
 		return 0;
 
 	*min = is->min_val;
 	*max = is->max_val;
-
-	n = (double) is->samples;
 	*mean = is->mean.u.f;
 
 	if (n > 1.0)
@@ -499,7 +497,8 @@ static void show_latencies(struct thread_stat *ts)
 	show_lat_m(io_u_lat_m);
 }
 
-void show_thread_status(struct thread_stat *ts, struct group_run_stats *rs)
+
+void show_thread_status_normal(struct thread_stat *ts, struct group_run_stats *rs)
 {
 	double usr_cpu, sys_cpu;
 	unsigned long runtime;
@@ -885,7 +884,8 @@ static void show_thread_status_terse_v3_v4(struct thread_stat *ts,
 		log_info(";%3.2f%%", io_u_lat_m[i]);
 
 	/* disk util stats, if any */
-	show_disk_util(1, NULL);
+	if (is_backend)
+		show_disk_util(1, NULL);
 
 	/* Additional output if continue_on_error set - default off*/
 	if (ts->continue_on_error)
@@ -972,7 +972,7 @@ static struct json_object *show_thread_status_json(struct thread_stat *ts,
 	/* Additional output if continue_on_error set - default off*/
 	if (ts->continue_on_error) {
 		json_object_add_value_int(root, "total_err", ts->total_err_count);
-		json_object_add_value_int(root, "total_err", ts->first_error);
+		json_object_add_value_int(root, "first_error", ts->first_error);
 	}
 
 	/* Additional output if description is set */
@@ -993,6 +993,18 @@ static void show_thread_status_terse(struct thread_stat *ts,
 		log_err("fio: bad terse version!? %d\n", terse_version);
 }
 
+struct json_object *show_thread_status(struct thread_stat *ts,
+				       struct group_run_stats *rs)
+{
+	if (output_format == FIO_OUTPUT_TERSE)
+		show_thread_status_terse(ts, rs);
+	else if (output_format == FIO_OUTPUT_JSON)
+		return(show_thread_status_json(ts, rs));
+	else
+		show_thread_status_normal(ts, rs);
+	return NULL;
+}
+
 static void sum_stat(struct io_stat *dst, struct io_stat *src, int nr)
 {
 	double mean, S;
@@ -1326,7 +1338,7 @@ static void __show_run_stats(void)
 			struct json_object *tmp = show_thread_status_json(ts, rs);
 			json_array_add_value_object(array, tmp);
 		} else
-			show_thread_status(ts, rs);
+			show_thread_status_normal(ts, rs);
 	}
 	if (output_format == FIO_OUTPUT_JSON) {
 		/* disk util stats, if any */
@@ -1506,13 +1518,23 @@ static void __add_log_sample(struct io_log *iolog, unsigned long val,
 {
 	const int nr_samples = iolog->nr_samples;
 
+	if (iolog->disabled)
+		return;
+
 	if (!iolog->nr_samples)
 		iolog->avg_last = t;
 
 	if (iolog->nr_samples == iolog->max_samples) {
 		int new_size = sizeof(struct io_sample) * iolog->max_samples*2;
+		void *new_log;
 
-		iolog->log = realloc(iolog->log, new_size);
+		new_log = realloc(iolog->log, new_size);
+		if (!new_log) {
+			log_err("fio: failed extending iolog! Will stop logging.\n");
+			iolog->disabled = 1;
+			return;
+		}
+		iolog->log = new_log;
 		iolog->max_samples <<= 1;
 	}
 
@@ -1693,7 +1715,7 @@ void add_bw_sample(struct thread_data *td, enum fio_ddir ddir, unsigned int bs,
 	fio_gettime(&td->bw_sample_time, NULL);
 }
 
-void add_iops_sample(struct thread_data *td, enum fio_ddir ddir,
+void add_iops_sample(struct thread_data *td, enum fio_ddir ddir, unsigned int bs,
 		     struct timeval *t)
 {
 	struct thread_stat *ts = &td->ts;
@@ -1720,7 +1742,7 @@ void add_iops_sample(struct thread_data *td, enum fio_ddir ddir,
 		add_stat_sample(&ts->iops_stat[ddir], iops);
 
 		if (td->iops_log)
-			add_log_sample(td, td->iops_log, iops, ddir, 0);
+			add_log_sample(td, td->iops_log, iops, ddir, bs);
 
 		td->stat_io_blocks[ddir] = td->this_io_blocks[ddir];
 	}
